@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2012,2015 Marc Alexander Lehmann <schmorp@schmorp.de>
+ * Copyright (c) 2001-2012 Marc Alexander Lehmann <schmorp@schmorp.de>
  *
  * Redistribution and use in source and binary forms, with or without modifica-
  * tion, are permitted provided that the following conditions are met:
@@ -79,14 +79,6 @@
  * 2012-12-05 experimental fiber backend (allocates stack twice).
  * 2012-12-07 API version 3 - add coro_stack_alloc/coro_stack_free.
  * 2012-12-21 valgrind stack registering was broken.
- * 2015-12-05 experimental asm be for arm7, based on a patch by Nick Zavaritsky.
- *            use __name__ for predefined symbols, as in libecb.
- *            enable guard pages on arm, aarch64 and mips.
- * 2016-08-27 try to disable _FORTIFY_SOURCE with CORO_SJLJ, as it
- *            breaks setjmp/longjmp. Also disable CORO_ASM for asm by default,
- *            as it was reported to crash.
- * 2016-11-18 disable cfi_undefined again - backtraces might be worse, but
- *            compile compatibility is improved.
  */
 
 #ifndef CORO_H
@@ -148,8 +140,8 @@ extern "C" {
  * -DCORO_ASM
  *
  *    Hand coded assembly, known to work only on a few architectures/ABI:
- *    GCC + arm7/x86/IA32/amd64/x86_64 + GNU/Linux and a few BSDs. Fastest
- *    choice, if it works.
+ *    GCC + x86/IA32 and amd64/x86_64 + GNU/Linux and a few BSDs. Fastest choice,
+ *    if it works.
  *
  * -DCORO_PTHREAD
  *
@@ -307,11 +299,11 @@ void coro_stack_free (struct coro_stack *stack);
     && !defined CORO_SJLJ    && !defined CORO_LINUX \
     && !defined CORO_IRIX    && !defined CORO_ASM \
     && !defined CORO_PTHREAD && !defined CORO_FIBER
-# if defined WINDOWS && (defined __i386__ || (__x86_64__ || defined _M_IX86 || defined _M_AMD64)
+# if defined WINDOWS && (defined __i386 || (__x86_64 || defined _M_IX86 || defined _M_AMD64)
 #  define CORO_ASM 1
 # elif defined WINDOWS || defined _WIN32
 #  define CORO_LOSER 1 /* you don't win with windoze */
-# elif __linux && (__i386__ || (__x86_64__ && !__ILP32__) /*|| (__arm__ && __ARM_ARCH == 7)), not working */
+# elif __linux && (__i386 || (__x86_64 && !__ILP32))
 #  define CORO_ASM 1
 # elif defined HAVE_UCONTEXT_H
 #  define CORO_UCONTEXT 1
@@ -342,12 +334,6 @@ struct coro_context
 #  define _GNU_SOURCE /* for glibc */
 # endif
 
-/* try to disable well-meant but buggy checks in some libcs */
-# ifdef _FORTIFY_SOURCE
-#  undef _FORTIFY_SOURCE
-#  undef __USE_FORTIFY_LEVEL /* helps some more when too much has been included already */
-# endif
-
 # if !CORO_LOSER
 #  include <unistd.h>
 # endif
@@ -389,11 +375,7 @@ struct coro_context
   void **sp; /* must be at offset 0 */
 };
 
-#if __i386__ || __x86_64__
 void __attribute__ ((__noinline__, __regparm__(2)))
-#else
-void __attribute__ ((__noinline__))
-#endif
 coro_transfer (coro_context *prev, coro_context *next);
 
 # define coro_destroy(ctx) (void *)(ctx)
